# SPDX-License-Identifier: AGPL-3.0-or-later
"""FreedomBox app to manage filesystem snapshots."""

import pathlib

import augeas
from django.utils.translation import gettext_lazy as _

from plinth import app as app_module
from plinth import menu
from plinth.modules import storage
from plinth.modules.backups.components import BackupRestore
from plinth.package import Packages

from . import manifest, privileged

_description = [
    _('Snapshots allows creating and managing btrfs file system snapshots. '
      'These can be used to roll back the system to a previously known '
      'good state in case of unwanted changes to the system.'),
    # Translators: xgettext:no-python-format
    _('Snapshots are taken periodically (called timeline snapshots) and also '
      'before and after a software installation. Older snapshots will be '
      'automatically cleaned up according to the settings below.'),
    _('Snapshots currently work on btrfs file systems only and on the root '
      'partition only. Snapshots are not a replacement for '
      '<a href="/plinth/sys/backups">backups</a> since '
      'they can only be stored on the same partition. ')
]

DEFAULT_FILE = '/etc/default/snapper'

fs_types_supported = ['btrfs']


class SnapshotApp(app_module.App):
    """FreedomBox app for snapshots."""

    app_id = 'snapshot'

    _version = 5

    can_be_disabled = False

    def __init__(self) -> None:
        """Create components for the app."""
        super().__init__()

        info = app_module.Info(app_id=self.app_id, version=self._version,
                               is_essential=True, name=_('Storage Snapshots'),
                               icon='fa-film', description=_description,
                               manual_page='Snapshots')
        self.add(info)

        menu_item = menu.Menu('menu-snapshot', info.name, None, info.icon,
                              'snapshot:index', parent_url_name='system:data',
                              order=40)
        self.add(menu_item)

        packages = Packages('packages-snapshot', ['snapper'])
        self.add(packages)

        backup_restore = SnapshotBackupRestore('backup-restore-snapshot',
                                               **manifest.backup)
        self.add(backup_restore)

    def setup(self, old_version):
        """Install and configure the app."""
        super().setup(old_version)
        if is_supported():
            privileged.setup(old_version)
            self.enable()


class SnapshotBackupRestore(BackupRestore):
    """Component to backup/restore snapshot module."""

    def restore_post(self, packet):
        """Run after restore."""
        privileged.kill_daemon()


def is_supported():
    """Return whether snapshots are support on current setup."""
    fs_type = storage.get_filesystem_type()
    # Check that / is not a bind mounted btrfs filesystem similar to how
    # snapper does the check: https://github.com/openSUSE/snapper/blob/
    # 77eb6565d3d8df95a06cd52ce31174d98994939c/snapper/BtrfsUtils.cc#L61
    root_inode_number = pathlib.Path('/').stat().st_ino
    return fs_type in fs_types_supported and root_inode_number == 256


def load_augeas():
    """Initialize Augeas."""
    aug = augeas.Augeas(flags=augeas.Augeas.NO_LOAD +
                        augeas.Augeas.NO_MODL_AUTOLOAD)

    # shell-script config file lens
    aug.set('/augeas/load/Shellvars/lens', 'Shellvars.lns')
    aug.set('/augeas/load/Shellvars/incl[last() + 1]', DEFAULT_FILE)
    aug.load()
    return aug


def is_apt_snapshots_enabled(aug):
    """Return whether APT snapshots is enabled."""
    value = aug.get('/files' + DEFAULT_FILE + '/DISABLE_APT_SNAPSHOT')
    return value != 'yes'


def get_configuration():
    """Return snapper configuration."""
    aug = load_augeas()
    output = privileged.get_config()

    def get_boolean_choice(status):
        return ('yes', 'Enabled') if status else ('no', 'Disabled')

    def get_max_from_range(key):
        return output[key].split('-')[-1]

    return {
        'enable_timeline_snapshots':
            get_boolean_choice(output['TIMELINE_CREATE'] == 'yes'),
        'enable_software_snapshots':
            get_boolean_choice(is_apt_snapshots_enabled(aug)),
        'hourly_limit':
            get_max_from_range('TIMELINE_LIMIT_HOURLY'),
        'daily_limit':
            get_max_from_range('TIMELINE_LIMIT_DAILY'),
        'weekly_limit':
            get_max_from_range('TIMELINE_LIMIT_WEEKLY'),
        'monthly_limit':
            get_max_from_range('TIMELINE_LIMIT_MONTHLY'),
        'yearly_limit':
            get_max_from_range('TIMELINE_LIMIT_YEARLY'),
        'free_space':
            output['FREE_LIMIT'],
    }



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Mumble server configuration form
"""

from django import forms
from django.utils.translation import gettext_lazy as _

from plinth.modules import mumble


def get_domain_choices():
    """Double domain entries for inclusion in the choice field."""
    return ((domain, domain) for domain in mumble.get_available_domains())


class MumbleForm(forms.Form):
    """Mumble server configuration"""
    domain = forms.ChoiceField(
        choices=get_domain_choices,
        label=_('TLS domain'),
        help_text=_(
            'Select a domain to use TLS with. If the list is empty, please '
            'configure at least one domain with certificates.'),
        required=False,
    )

    super_user_password = forms.CharField(
        max_length=20,
        label=_('Set SuperUser Password'),
        widget=forms.PasswordInput,
        help_text=_(
            'Optional. Leave this field blank to keep the current password. '
            'SuperUser password can be used to manage permissions in Mumble.'),
        required=False,
    )

    join_password = forms.CharField(
        max_length=32,
        label=_('Set a password to join the server'),
        widget=forms.PasswordInput,
        help_text=_('Set a password that is required to join the server. '
                    'Leave empty to use the current password.'),
        required=False,
    )

    root_channel_name = forms.CharField(
        label=_('Set the name for the root channel'),
        min_length=1,
        max_length=32,
        help_text=_(
            'Set the name of the main channel of your mumble server. '
            'If the name was never changed, the channel is named Root.'),
        required=False,
    )



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Django URL patterns for running tests.
"""

from django.urls import re_path
from django.views.generic import TemplateView

_test_view = TemplateView.as_view(template_name='index.html')
urlpatterns = [
    re_path(r'^$', _test_view, name='index'),
    re_path(r'^apps/$', _test_view, name='apps'),
    re_path(r'^sys/$', _test_view, name='system'),
    re_path(r'^test/(?P<a>\d+)/(?P<b>\d+)/(?P<c>\d+)/$', _test_view,
            name='test'),
]



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Tests for form field validators in backups.
"""

import pytest
from django.core.exceptions import ValidationError

from .. import split_path
from ..forms import repository_validator


def _validate_repository(valid_list, invalid_list, path_string):
    """Assert that repository strings in given list (in)validate properly."""
    for item in valid_list:
        repository_validator(path_string.format(item))

    for item in invalid_list:
        path = path_string.format(item)
        with pytest.raises(ValidationError):
            repository_validator(path)


def test_repository_paths_validation():
    """Test that repository strings are validated properly."""
    valid_paths = ['sshuser@10.0.2.2:~/backups']
    invalid_paths = [
        'mary had a little lamb', 'someone@example.com', 'www.example.com',
        'sshuser@hostname'
    ]
    path_string = '{}'
    _validate_repository(valid_paths, invalid_paths, path_string)


def test_repository_username_validation():
    """Test that usernames in repository string are validated properly."""
    valid_usernames = [
        'sshuser', 'cypher_punk-2077', '_user', '_-_', '1two', '1234'
    ]
    invalid_usernames = ['somebody else']
    path_string = '{}@example.org:~/backups'
    _validate_repository(valid_usernames, invalid_usernames, path_string)


def test_repository_hostname_validation():
    """Test that hostnames in repository string are validated properly."""
    valid_hostnames = [
        '192.168.0.1', 'fe80::2078:6c26:498a:1fa5%wlps0', 'freedombox.org',
        '1.percent.org', 'freedombox', '::1'
    ]
    invalid_hostnames = ['192.fe80::2089:1fa5']
    path_string = 'user@{}:~/backups'
    _validate_repository(valid_hostnames, invalid_hostnames, path_string)


def test_repository_dir_path_validation():
    """Test that paths in repository string are validated properly."""
    valid_dir_paths = [
        '~/backups', '/home/user/backup-folder_1/', '', '/foo:bar/'
    ]
    invalid_dir_paths = []
    path_string = 'user@localhost:{}'
    _validate_repository(valid_dir_paths, invalid_dir_paths, path_string)


def test_repository_with_colon_path():
    """Test that a colon is possible in directory path."""
    _, hostname, path = split_path('user@fe80::2078:6c26:498a:1fa5:/foo:bar')
    assert hostname == 'fe80::2078:6c26:498a:1fa5'
    assert path == '/foo:bar'



# SPDX-License-Identifier: AGPL-3.0-or-later
"""Tor utility functions."""

from plinth import app as app_module
from plinth.daemon import app_is_running
from plinth.modules.names.components import DomainName

from . import privileged


def get_status(initialized=True):
    """Return current Tor status."""
    status = privileged.get_status()

    hs_info = status['hidden_service']
    hs_services = []
    if hs_info['hostname']:
        try:
            domain = DomainName.get('domain-tor-' + hs_info['hostname'])
        except KeyError:
            pass
        else:
            hs_services = domain.get_readable_services()

    # Filter out obfs3/4 ports when bridge relay is disabled
    ports = {
        service_type: port
        for service_type, port in status['ports'].items()
        if service_type not in ['obfs4', 'obfs3']
        or status['bridge_relay_enabled']
    }

    app = app_module.App.get('tor')
    return {
        'enabled': app.is_enabled() if initialized else False,
        'is_running': app_is_running(app) if initialized else False,
        'use_upstream_bridges': status['use_upstream_bridges'],
        'upstream_bridges': status['upstream_bridges'],
        'relay_enabled': status['relay_enabled'],
        'bridge_relay_enabled': status['bridge_relay_enabled'],
        'ports': ports,
        'hs_enabled': hs_info['enabled'],
        'hs_status': hs_info['status'],
        'hs_hostname': hs_info['hostname'],
        'hs_ports': hs_info['ports'],
        'hs_services': hs_services,
    }



# SPDX-License-Identifier: AGPL-3.0-or-later
"""FreedomBox app for configuring date and time."""

import logging
import subprocess

from django.contrib import messages
from django.utils.translation import gettext as _

from plinth.views import AppView

from . import privileged
from .forms import DateTimeForm

logger = logging.getLogger(__name__)


class DateTimeAppView(AppView):
    """Serve configuration page."""

    form_class = DateTimeForm
    app_id = 'datetime'

    def get_initial(self):
        """Return the values to fill in the form."""
        status = super().get_initial()
        status['time_zone'] = self.get_current_time_zone() or 'none'
        return status

    @staticmethod
    def get_current_time_zone():
        """Get current time zone."""
        try:
            process = subprocess.run(
                ['timedatectl', 'show', '--property', 'Timezone', '--value'],
                stdout=subprocess.PIPE, check=True)
            return process.stdout.decode().strip()
        except subprocess.CalledProcessError:
            return None

    def form_valid(self, form):
        """Change the timezone."""
        old_status = form.initial
        new_status = form.cleaned_data

        if old_status['time_zone'] != new_status['time_zone'] and \
           new_status['time_zone'] != 'none':
            try:
                privileged.set_timezone(new_status['time_zone'])
            except Exception as exception:
                messages.error(
                    self.request,
                    _('Error setting time zone: {exception}').format(
                        exception=exception))
            else:
                messages.success(self.request, _('Time zone set'))

        return super().form_valid(form)



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
URLs for the First Boot module
"""

from django.urls import re_path
from stronghold.decorators import public

from .views import CompleteView, WelcomeView

urlpatterns = [
    # Take care of the firstboot middleware when changing URLs
    re_path(r'^firstboot/$', public(WelcomeView.as_view()), name='index'),
    re_path(r'^firstboot/welcome/$', public(WelcomeView.as_view()),
            name='welcome'),
    re_path(r'^firstboot/complete/$', CompleteView.as_view(), name='complete'),
]



# SPDX-License-Identifier: AGPL-3.0-or-later
"""Configure postix.

- Configure postfix to use auth and local delivery with dovecot.
- Start SMTPS and submission services. Setup aliases database.

See:
https://doc.dovecot.org/configuration_manual/howto/postfix_and_dovecot_sasl/
See: https://doc.dovecot.org/configuration_manual/howto/postfix_dovecot_lmtp/
See: http://www.postfix.org/TLS_README.html
"""

from plinth.actions import privileged

from .. import postfix as postconf

default_config = {
    'smtpd_sasl_auth_enable':
        'yes',
    'smtpd_sasl_type':
        'dovecot',
    'smtpd_sasl_path':
        'private/auth',
    'mailbox_transport':
        'lmtp:unix:private/dovecot-lmtp',
    'virtual_transport':
        'lmtp:unix:private/dovecot-lmtp',
    'smtpd_relay_restrictions':
        ','.join([
            'permit_sasl_authenticated',
            'defer_unauth_destination',
        ]),
    'message_size_limit':
        '102400000',
}

submission_options: dict[str, str] = {
    'syslog_name': 'postfix/submission',
    'smtpd_tls_security_level': 'encrypt',
    'smtpd_client_restrictions': 'permit_sasl_authenticated,reject',
    'smtpd_relay_restrictions': 'permit_sasl_authenticated,reject'
}
submission_service = postconf.Service(service='submission', type_='inet',
                                      private='n', unpriv='-', chroot='y',
                                      wakeup='-', maxproc='-', command='smtpd',
                                      options=submission_options)

smtps_options: dict[str, str] = {
    'syslog_name': 'postfix/smtps',
    'smtpd_tls_wrappermode': 'yes',
    'smtpd_sasl_auth_enable': 'yes',
    'smtpd_relay_restrictions': 'permit_sasl_authenticated,reject'
}
smtps_service = postconf.Service(service='smtps', type_='inet', private='n',
                                 unpriv='-', chroot='y', wakeup='-',
                                 maxproc='-', command='smtpd',
                                 options=smtps_options)

SQLITE_ALIASES = 'sqlite:/etc/postfix/freedombox-aliases.cf'


@privileged
def setup_postfix():
    """Configure postfix."""
    postconf.set_config(default_config)
    _setup_submission()
    _setup_alias_maps()


def _setup_submission():
    """Update configuration for smtps and smtp-submission."""
    postconf.set_master_config(submission_service)
    postconf.set_master_config(smtps_service)


def _setup_alias_maps():
    """Setup alias maps to include an sqlite DB."""
    alias_maps = postconf.get_config(['alias_maps'])['alias_maps']
    alias_maps = alias_maps.replace(',', ' ').split(' ')
    if SQLITE_ALIASES not in alias_maps:
        # Prioritize FreedomBox's sqlite based aliases file over /etc/aliases.
        # Otherwise, the common aliases will be pointing to 'root' instead of
        # first admin user (which is more practical in FreedomBox).
        alias_maps = [SQLITE_ALIASES] + alias_maps

    postconf.set_config({'alias_maps': ' '.join(alias_maps)})



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Functional, browser based tests for pagekite app.
"""

import pytest
from plinth.tests import functional

pytestmark = [pytest.mark.system, pytest.mark.domain, pytest.mark.pagekite]

# TODO Scenario: Enable standard services
# TODO Scenario: Disable standard services
# TODO Scenario: Add custom service
# TODO Scenario: Delete custom service


class TestPagekiteApp(functional.BaseAppTests):
    app_name = 'pagekite'
    has_service = True
    has_web = False

    def test_configure(self, session_browser):
        """Test pagekite configuration."""
        functional.app_enable(session_browser, 'pagekite')
        _configure(session_browser, 'pagekite.example.com', 8080,
                   'mykite.example.com', 'mysecret')
        assert ('pagekite.example.com', 8080, 'mykite.example.com',
                'mysecret') == _get_configuration(session_browser)

        # Capitalized kite name should become lower case.
        _configure(session_browser, 'pagekite.example.com', 8080,
                   'Mykite.example.com', 'mysecret')
        assert ('pagekite.example.com', 8080, 'mykite.example.com',
                'mysecret') == _get_configuration(session_browser)

    @pytest.mark.backups
    def test_backup_restore(self, session_browser):
        """Test backup and restore of configuration."""
        functional.app_enable(session_browser, 'pagekite')
        _configure(session_browser, 'beforebackup.example.com', 8081,
                   'beforebackup.example.com', 'beforebackupsecret')
        functional.backup_create(session_browser, 'pagekite', 'test_pagekite')

        _configure(session_browser, 'afterbackup.example.com', 8082,
                   'afterbackup.example.com', 'afterbackupsecret')
        functional.backup_restore(session_browser, 'pagekite', 'test_pagekite')

        assert functional.service_is_running(session_browser, 'pagekite')
        assert ('beforebackup.example.com', 8081, 'beforebackup.example.com',
                'beforebackupsecret') == _get_configuration(session_browser)


def _configure(browser, host, port, kite_name, kite_secret):
    """Configure pagekite basic parameters."""
    functional.nav_to_module(browser, 'pagekite')
    # time.sleep(0.250)  # Wait for 200ms show animation to complete
    browser.fill('pagekite-server_domain', host)
    browser.fill('pagekite-server_port', str(port))
    browser.fill('pagekite-kite_name', kite_name)
    browser.fill('pagekite-kite_secret', kite_secret)
    functional.submit(browser, form_class='form-configuration')


def _get_configuration(browser):
    """Return pagekite basic parameters."""
    functional.nav_to_module(browser, 'pagekite')
    return (browser.find_by_name('pagekite-server_domain').value,
            int(browser.find_by_name('pagekite-server_port').value),
            browser.find_by_name('pagekite-kite_name').value,
            browser.find_by_name('pagekite-kite_secret').value)



# SPDX-License-Identifier: AGPL-3.0-or-later
"""FreedomBox app for configuring Shadowsocks Server."""

from django import forms
from django.utils.translation import gettext_lazy as _

from plinth.utils import format_lazy

METHODS = [('chacha20-ietf-poly1305',
            format_lazy('chacha20-ietf-poly1305 ({})', _('Recommended'))),
           ('aes-256-gcm', format_lazy('aes-256-gcm ({})', _('Recommended'))),
           ('aes-192-gcm', 'aes-192-gcm'), ('aes-128-gcm', 'aes-128-gcm'),
           ('aes-128-ctr', 'aes-128-ctr'), ('aes-192-ctr', 'aes-192-ctr'),
           ('aes-256-ctr', 'aes-256-ctr'), ('aes-128-cfb', 'aes-128-cfb'),
           ('aes-192-cfb', 'aes-192-cfb'), ('aes-256-cfb', 'aes-256-cfb'),
           ('camellia-128-cfb', 'camellia-128-cfb'),
           ('camellia-192-cfb', 'camellia-192-cfb'),
           ('camellia-256-cfb', 'camellia-256-cfb'),
           ('chacha20-ietf', 'chacha20-ietf')]


class TrimmedCharField(forms.CharField):
    """Trim the contents of a CharField."""

    def clean(self, value):
        """Clean and validate the field value."""
        if value:
            value = value.strip()

        return super().clean(value)


class ShadowsocksServerForm(forms.Form):
    """Shadowsocks Server configuration form."""

    password = forms.CharField(
        label=_('Password'),
        help_text=_('Password used to encrypt data. Clients must use the '
                    'same password.'))

    method = forms.ChoiceField(
        label=_('Method'), choices=METHODS,
        help_text=_('Encryption method. Clients must use the same setting.'))



# SPDX-License-Identifier: AGPL-3.0-or-later
"""FreedomBox app to configure Zoph web application."""

import logging

from django.utils.translation import gettext_lazy as _

from plinth import app as app_module
from plinth import cfg, frontpage, menu
from plinth.config import DropinConfigs
from plinth.daemon import SharedDaemon
from plinth.modules.apache.components import Webserver
from plinth.modules.backups.components import BackupRestore
from plinth.modules.firewall.components import Firewall
from plinth.package import Packages
from plinth.utils import format_lazy

from . import manifest, privileged

logger = logging.getLogger(__name__)

_description = [
    format_lazy(
        _('Zoph manages your photo collection. Photos are stored on your '
          '{box_name}, under your control. Instead of focusing on galleries '
          'for public display, Zoph focuses on managing them for your own '
          'use, organizing them by who took them, where they were taken, '
          'and who is in them. Photos can be linked to multiple hierarchical '
          'albums and categories. It is easy to find all photos containing a '
          'person, or photos taken on a date, or photos taken at a location '
          'using search, map and calendar views. Individual photos can be '
          'shared with others by sending a direct link.'),
        box_name=_(cfg.box_name)),
    format_lazy(
        _('The {box_name} user who setup Zoph will also become the '
          'administrator in Zoph. For additional users, accounts must be '
          'created both in {box_name} and in Zoph with the same user name.'),
        box_name=_(cfg.box_name))
]


class ZophApp(app_module.App):
    """FreedomBox app for Zoph."""

    app_id = 'zoph'

    _version = 2

    configure_when_disabled = False

    def __init__(self) -> None:
        """Create components for the app."""
        super().__init__()

        info = app_module.Info(app_id=self.app_id, version=self._version,
                               name=_('Zoph'), icon_filename='zoph',
                               short_description=_('Photo Organizer'),
                               description=_description, manual_page='Zoph',
                               clients=manifest.clients)
        self.add(info)

        menu_item = menu.Menu('menu-zoph', info.name, info.short_description,
                              info.icon_filename, 'zoph:index',
                              parent_url_name='apps')
        self.add(menu_item)

        shortcut = frontpage.Shortcut('shortcut-zoph', info.name,
                                      short_description=info.short_description,
                                      icon=info.icon_filename, url='/zoph/',
                                      clients=info.clients,
                                      login_required=True)
        self.add(shortcut)

        packages = Packages('packages-zoph', ['zoph', 'default-mysql-server'],
                            conflicts=['libpam-tmpdir'],
                            conflicts_action=Packages.ConflictsAction.REMOVE)
        self.add(packages)

        firewall = Firewall('firewall-zoph', info.name,
                            ports=['http', 'https'], is_external=True)
        self.add(firewall)

        dropin_configs = DropinConfigs('dropin-configs-zoph', [
            '/etc/apache2/conf-available/zoph-freedombox.conf',
        ])
        self.add(dropin_configs)

        webserver = Webserver('webserver-zoph', 'zoph',
                              urls=['https://{host}/zoph/'])
        self.add(webserver)

        webserver = Webserver('webserver-zoph-freedombox', 'zoph-freedombox')
        self.add(webserver)

        daemon = SharedDaemon('shared-daemon-zoph-mysql', 'mysql')
        self.add(daemon)

        backup_restore = ZophBackupRestore('backup-restore-zoph',
                                           **manifest.backup)
        self.add(backup_restore)

    def setup(self, old_version):
        """Install and configure the app."""
        privileged.pre_install()
        with self.get_component('shared-daemon-zoph-mysql').ensure_running():
            # Database needs to be running for successful initialization or
            # upgrade of zoph database.
            super().setup(old_version)

        privileged.setup()
        if not old_version:
            self.enable()
        elif old_version < 2:
            if self.get_component('webserver-zoph').is_enabled():
                self.enable()

    def uninstall(self):
        """De-configure and uninstall the app."""
        # Before package uninstall, so that config file is still available
        privileged.uninstall()
        super().uninstall()


class ZophBackupRestore(BackupRestore):
    """Component to backup/restore Zoph database"""

    def backup_pre(self, packet):
        """Save database contents."""
        super().backup_pre(packet)
        privileged.dump_database()

    def restore_post(self, packet):
        """Restore database contents."""
        super().restore_post(packet)
        privileged.restore_database()



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Configuration helper for Transmission daemon.
"""

import json
import pathlib

from plinth import action_utils
from plinth.actions import privileged

_transmission_config = pathlib.Path('/etc/transmission-daemon/settings.json')


@privileged
def get_configuration() -> dict[str, str]:
    """Return the current configuration in JSON format."""
    return json.loads(_transmission_config.read_text(encoding='utf-8'))


@privileged
def merge_configuration(configuration: dict[str, str | bool]):
    """Merge given JSON configuration with existing configuration."""
    current_configuration_bytes = _transmission_config.read_bytes()
    current_configuration = json.loads(current_configuration_bytes)

    new_configuration = current_configuration
    new_configuration.update(configuration)
    new_configuration_bytes = json.dumps(new_configuration, indent=4,
                                         sort_keys=True)

    _transmission_config.write_text(new_configuration_bytes, encoding='utf-8')
    action_utils.service_reload('transmission-daemon')



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Configuration for running tests.

To customize these settings, create a 'config_local.py' and override
the variables defined here.
"""

# When credentials are given, backups_ssh_path will be mounted.  In the given
# folder, repositories will be created in subfolders with random uuids.
backups_ssh_path = None
# provide backups_ssh_path and either a password or a keyfile for ssh tests
backups_ssh_password = None
backups_ssh_keyfile = None
backups_ssh_repo_uuid = 'plinth_test_sshfs'  # will be mounted to /media/<uuid>

# An existing admin account for privileged actions. If this admin account
# doesn't exist and no other admin accounts exist, a random admin account is
# created and deleted afterwards by the tests.
admin_username = 'tester'
admin_password = 'testingtesting'

# Import config_local to override the default variables
try:
    from .config_local import *  # noqa, pylint: disable=unused-import
except ImportError:
    pass



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Forms for minetest module.
"""

from django import forms
from django.utils.translation import gettext_lazy as _


class MinetestForm(forms.Form):
    """Minetest configuration form"""
    max_players = forms.IntegerField(
        label=_('Maximum number of players'), required=True, min_value=1,
        max_value=100,
        help_text=_('You can change the maximum number of players playing '
                    'minetest at a single instance of time.'))

    creative_mode = forms.BooleanField(
        label=_('Enable creative mode'), required=False,
        help_text=_('Creative mode changes the rules of the game to make it '
                    'more suitable for creative gameplay, rather than '
                    'challenging "survival" gameplay.'))

    enable_pvp = forms.BooleanField(
        label=_('Enable PVP'), required=False,
        help_text=_('Enabling Player Vs Player will allow players to damage '
                    'other players.'))

    enable_damage = forms.BooleanField(
        label=_('Enable damage'), required=False,
        help_text=_('When disabled, players cannot die or receive damage of '
                    'any kind.'))



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Functional, browser based tests for ssh app.
"""

import pytest

from plinth.tests.functional import BaseAppTests

pytestmark = [pytest.mark.system, pytest.mark.ssh]


class TestSshApp(BaseAppTests):
    app_name = 'ssh'
    has_service = True
    has_web = False
    disable_after_tests = False

    # TODO: Improve test_backup_restore to actually check that earlier
    # ssh certificate has been restored.

    def test_enable_disable(self, session_browser):
        """Skip test for enabling and disabling the app."""
        pytest.skip('Avoid restarting SSH server')

    def test_backup_restore(self, session_browser):
        """Skip test for backup and restore operations."""
        pytest.skip('Avoid restarting SSH server')



# SPDX-License-Identifier: AGPL-3.0-or-later

from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.translation import gettext as _
from django.views.generic import View
from django.views.generic.edit import FormView

from plinth.views import AppView

from . import privileged
from .forms import (AddCustomServiceForm, ConfigurationForm,
                    DeleteCustomServiceForm)


class DeleteServiceView(View):

    def post(self, request):
        form = DeleteCustomServiceForm(request.POST)
        if form.is_valid():
            form.delete(request)
        return HttpResponseRedirect(reverse('pagekite:index'))


class AddCustomServiceView(FormView):
    """View to add a new custom PageKite service."""
    form_class = AddCustomServiceForm
    template_name = 'pagekite_custom_services.html'
    success_url = reverse_lazy('pagekite:index')

    def get_context_data(self, *args, **kwargs):
        """Return additional context for rendering the template."""
        context = super().get_context_data(*args, **kwargs)
        context['title'] = _('Add custom PageKite service')
        return context

    def form_valid(self, form):
        """Add a custom service."""
        form.save(self.request)
        return super().form_valid(form)


class ConfigurationView(AppView):
    app_id = 'pagekite'
    template_name = 'pagekite_configure.html'
    form_class = ConfigurationForm
    prefix = 'pagekite'
    success_url = reverse_lazy('pagekite:index')

    def __init__(self, *args, **kwargs):
        """Load and store the current configuration."""
        super().__init__(*args, **kwargs)
        self.config = privileged.get_config()
        self.initial = self.config

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        for service in self.config['custom_services']:
            service['delete_form'] = DeleteCustomServiceForm(initial=service)
        context.update(self.config)
        return context

    def form_valid(self, form):
        form.save(self.request)
        return super().form_valid(form)



# SPDX-License-Identifier: AGPL-3.0-or-later
"""Views for the minidlna module."""

from django.contrib import messages
from django.utils.translation import gettext_lazy as _

from plinth.views import AppView

from . import privileged
from .forms import MiniDLNAServerForm


class MiniDLNAAppView(AppView):
    """Show minidlna app view."""

    app_id = 'minidlna'
    form_class = MiniDLNAServerForm

    def get_initial(self):
        """Return initial values of the form."""
        initial = super().get_initial()
        initial.update({'storage_path': privileged.get_media_dir()})

        return initial

    def form_valid(self, form):
        """Apply changes from the form."""
        old_config = form.initial
        new_config = form.cleaned_data

        if old_config['storage_path'] != new_config['storage_path']:
            privileged.set_media_dir(new_config['storage_path'])
            messages.success(self.request, _('Updated media directory'))

        return super().form_valid(form)
